/***********************************************************************
 *
 * CONFIG.C - Nagios Configuration CGI (View Only)
 *
 * Copyright (c) 1999-2008 Ethan Galstad (nagios@nagios.org)
 * Last Modified: 03-31-2008
 *
 * This CGI program will display various configuration information.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ***********************************************************************/

#include "../include/config.h"
#include "../include/common.h"
#include "../include/objects.h"
#include "../include/macros.h"
#include "../include/cgiutils.h"
#include "../include/cgiauth.h"
#include "../include/getcgi.h"

extern char   main_config_file[MAX_FILENAME_LENGTH];
extern char   url_html_path[MAX_FILENAME_LENGTH];
extern char   url_docs_path[MAX_FILENAME_LENGTH];
extern char   url_images_path[MAX_FILENAME_LENGTH];
extern char   url_logo_images_path[MAX_FILENAME_LENGTH];
extern char   url_stylesheets_path[MAX_FILENAME_LENGTH];

extern host *host_list;
extern service *service_list;
extern hostgroup *hostgroup_list;
extern servicegroup *servicegroup_list;
extern contactgroup *contactgroup_list;
extern command *command_list;
extern timeperiod *timeperiod_list;
extern contact *contact_list;
extern servicedependency *servicedependency_list;
extern serviceescalation *serviceescalation_list;
extern hostdependency *hostdependency_list;
extern hostescalation *hostescalation_list;


#define DISPLAY_NONE                     0
#define DISPLAY_HOSTS                    1
#define DISPLAY_HOSTGROUPS               2
#define DISPLAY_CONTACTS                 3
#define DISPLAY_CONTACTGROUPS            4
#define DISPLAY_SERVICES                 5
#define DISPLAY_TIMEPERIODS              6
#define DISPLAY_COMMANDS                 7
#define DISPLAY_HOSTGROUPESCALATIONS     8    /* no longer implemented */
#define DISPLAY_SERVICEDEPENDENCIES      9
#define DISPLAY_SERVICEESCALATIONS       10
#define DISPLAY_HOSTDEPENDENCIES         11
#define DISPLAY_HOSTESCALATIONS          12
#define DISPLAY_SERVICEGROUPS            15

void document_header(int);
void document_footer(void);
int process_cgivars(void);

void display_options(void);

void display_hosts(void);
void display_hostgroups(void);
void display_servicegroups(void);
void display_contacts(void);
void display_contactgroups(void);
void display_services(void);
void display_timeperiods(void);
void display_commands(void);
void display_servicedependencies(void);
void display_serviceescalations(void);
void display_hostdependencies(void);
void display_hostescalations(void);

void unauthorized_message(void);


authdata current_authdata;

int display_type=DISPLAY_NONE;

int embedded=FALSE;


int main(void){
	int result=OK;

	/* get the arguments passed in the URL */
	process_cgivars();

	/* reset internal variables */
	reset_cgi_vars();

	/* read the CGI configuration file */
	result=read_cgi_config_file(get_cgi_config_location());
	if(result==ERROR){
		document_header(FALSE);
		cgi_config_file_error(get_cgi_config_location());
		document_footer();
		return ERROR;
	        }

	/* read the main configuration file */
	result=read_main_config_file(main_config_file);
	if(result==ERROR){
		document_header(FALSE);
		main_config_file_error(main_config_file);
		document_footer();
		return ERROR;
	        }

	/* read all object configuration data */
	result=read_all_object_configuration_data(main_config_file,READ_ALL_OBJECT_DATA);
	if(result==ERROR){
		document_header(FALSE);
		object_data_error();
		document_footer();
		return ERROR;
                }

	/* initialize macros */
	init_macros();

	document_header(TRUE);

	/* get authentication information */
	get_authentication_information(&current_authdata);

	/* read all object definitions... */
	read_all_object_configuration_data(main_config_file,READ_ALL_OBJECT_DATA);

	/* begin top table */
	printf("<table border=0 width=100%%>\n");
	printf("<tr>\n");

	/* left column of the first row */
	printf("<td align=left valign=top width=50%%>\n");
	display_info_table("各種設定-Configuration-",FALSE,&current_authdata);
	printf("</td>\n");

	/* right hand column of top row */
	printf("<td align=right valign=bottom width=50%%>\n");

	if(display_type!=DISPLAY_NONE){

		printf("<form method=\"get\" action=\"%s\">\n",CONFIG_CGI);
		printf("<table border=0>\n");

		printf("<tr><td align=left class='reportSelectSubTitle'>オブジェクトタイプ:</td></tr>\n");
		printf("<tr><td align=left class='reportSelectItem'>");
		printf("<select name='type'>\n");
		printf("<option value='hosts' %s>ホスト\n",(display_type==DISPLAY_HOSTS)?"SELECTED":"");
		printf("<option value='hostdependencies' %s>ホスト依存関係\n",(display_type==DISPLAY_HOSTDEPENDENCIES)?"SELECTED":"");
		printf("<option value='hostescalations' %s>ホストエスカレーション\n",(display_type==DISPLAY_HOSTESCALATIONS)?"SELECTED":"");
		printf("<option value='hostgroups' %s>ホストグループ\n",(display_type==DISPLAY_HOSTGROUPS)?"SELECTED":"");
		printf("<option value='services' %s>サービス\n",(display_type==DISPLAY_SERVICES)?"SELECTED":"");
		printf("<option value='servicegroups' %s>サービスグループ\n",(display_type==DISPLAY_SERVICEGROUPS)?"SELECTED":"");
		printf("<option value='servicedependencies' %s>サービス依存関係\n",(display_type==DISPLAY_SERVICEDEPENDENCIES)?"SELECTED":"");
		printf("<option value='serviceescalations' %s>サービスエスカレーション\n",(display_type==DISPLAY_SERVICEESCALATIONS)?"SELECTED":"");
		printf("<option value='contacts' %s>通知先\n",(display_type==DISPLAY_CONTACTS)?"SELECTED":"");
		printf("<option value='contactgroups' %s>通知先グループ\n",(display_type==DISPLAY_CONTACTGROUPS)?"SELECTED":"");
		printf("<option value='timeperiods' %s>時間帯\n",(display_type==DISPLAY_TIMEPERIODS)?"SELECTED":"");
		printf("<option value='commands' %s>コマンド\n",(display_type==DISPLAY_COMMANDS)?"SELECTED":"");
		printf("</select>\n");
		printf("</td></tr>\n");

		printf("<tr><td class='reportSelectItem'><input type='submit' value='更新'></td></tr>\n");
		printf("</table>\n");
		printf("</form>\n");
	        }

	/* display context-sensitive help */
	switch(display_type){
	case DISPLAY_HOSTS:
		display_context_help(CONTEXTHELP_CONFIG_HOSTS);
		break;
	case DISPLAY_HOSTGROUPS:
		display_context_help(CONTEXTHELP_CONFIG_HOSTGROUPS);
		break;
	case DISPLAY_SERVICEGROUPS:
		display_context_help(CONTEXTHELP_CONFIG_SERVICEGROUPS);
		break;
	case DISPLAY_CONTACTS:
		display_context_help(CONTEXTHELP_CONFIG_CONTACTS);
		break;
	case DISPLAY_CONTACTGROUPS:
		display_context_help(CONTEXTHELP_CONFIG_CONTACTGROUPS);
		break;
	case DISPLAY_SERVICES:
		display_context_help(CONTEXTHELP_CONFIG_SERVICES);
		break;
	case DISPLAY_TIMEPERIODS:
		display_context_help(CONTEXTHELP_CONFIG_TIMEPERIODS);
		break;
	case DISPLAY_COMMANDS:
		display_context_help(CONTEXTHELP_CONFIG_COMMANDS);
		break;
	case DISPLAY_SERVICEDEPENDENCIES:
		display_context_help(CONTEXTHELP_CONFIG_SERVICEDEPENDENCIES);
		break;
	case DISPLAY_SERVICEESCALATIONS:
		display_context_help(CONTEXTHELP_CONFIG_HOSTESCALATIONS);
		break;
	case DISPLAY_HOSTDEPENDENCIES:
		display_context_help(CONTEXTHELP_CONFIG_HOSTDEPENDENCIES);
		break;
	case DISPLAY_HOSTESCALATIONS:
		display_context_help(CONTEXTHELP_CONFIG_HOSTESCALATIONS);
		break;
	default:
		display_context_help(CONTEXTHELP_CONFIG_MENU);
		break;
	        }

	printf("</td>\n");

	/* end of top table */
	printf("</tr>\n");
	printf("</table>\n");


	switch(display_type){
	case DISPLAY_HOSTS:
		display_hosts();
		break;
	case DISPLAY_HOSTGROUPS:
		display_hostgroups();
		break;
	case DISPLAY_SERVICEGROUPS:
		display_servicegroups();
		break;
	case DISPLAY_CONTACTS:
		display_contacts();
		break;
	case DISPLAY_CONTACTGROUPS:
		display_contactgroups();
		break;
	case DISPLAY_SERVICES:
		display_services();
		break;
	case DISPLAY_TIMEPERIODS:
		display_timeperiods();
		break;
	case DISPLAY_COMMANDS:
		display_commands();
		break;
	case DISPLAY_SERVICEDEPENDENCIES:
		display_servicedependencies();
		break;
	case DISPLAY_SERVICEESCALATIONS:
		display_serviceescalations();
		break;
	case DISPLAY_HOSTDEPENDENCIES:
		display_hostdependencies();
		break;
	case DISPLAY_HOSTESCALATIONS:
		display_hostescalations();
		break;
	default:
		display_options();
		break;
	        }

	document_footer();

	return OK;
        }




void document_header(int use_stylesheet){
	char date_time[MAX_DATETIME_LENGTH];
	time_t t;

	if(embedded==TRUE)
		return;

	time(&t);
	get_time_string(&t,date_time,sizeof(date_time),HTTP_DATE_TIME);

	printf("Cache-Control: no-store\r\n");
	printf("Pragma: no-cache\r\n");
	printf("Last-Modified: %s\r\n",date_time);
	printf("Expires: %s\r\n",date_time);
	printf("Content-type: text/html; Charset=UTF-8\r\n\r\n");

	printf("<html>\n");
	printf("<head>\n");
	printf("<link rel=\"shortcut icon\" href=\"%sfavicon.ico\" type=\"image/ico\">\n",url_images_path);
	printf("<META HTTP-EQUIV='Pragma' CONTENT='no-cache'>\n");
	printf("<meta http-equiv='Content-Type' content='text/html; Charset=UTF-8'>\n");
	printf("<title>\n");
	printf("設定\n");
	printf("</title>\n");

	if(use_stylesheet==TRUE){
		printf("<LINK REL='stylesheet' TYPE='text/css' HREF='%s%s'>\n",url_stylesheets_path,COMMON_CSS);
		printf("<LINK REL='stylesheet' TYPE='text/css' HREF='%s%s'>\n",url_stylesheets_path,CONFIG_CSS);
	        }

	printf("</head>\n");

	printf("<body CLASS='config'>\n");

	/* include user SSI header */
	include_ssi_files(CONFIG_CGI,SSI_HEADER);

	return;
        }


void document_footer(void){

	if(embedded==TRUE)
		return;

	/* include user SSI footer */
	include_ssi_files(CONFIG_CGI,SSI_FOOTER);

	printf("</body>\n");
	printf("</html>\n");

	return;
        }


int process_cgivars(void){
	char **variables;
	int error=FALSE;
	int x;

	variables=getcgivars();

	for(x=0;variables[x]!=NULL;x++){

		/* do some basic length checking on the variable identifier to prevent buffer overflows */
		if(strlen(variables[x])>=MAX_INPUT_BUFFER-1){
			x++;
			continue;
		        }

		/* we found the configuration type argument */
		else if(!strcmp(variables[x],"type")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			/* what information should we display? */
			if(!strcmp(variables[x],"hosts"))
				display_type=DISPLAY_HOSTS;
			else if(!strcmp(variables[x],"hostgroups"))
				display_type=DISPLAY_HOSTGROUPS;
			else if(!strcmp(variables[x],"servicegroups"))
				display_type=DISPLAY_SERVICEGROUPS;
			else if(!strcmp(variables[x],"contacts"))
				display_type=DISPLAY_CONTACTS;
			else if(!strcmp(variables[x],"contactgroups"))
				display_type=DISPLAY_CONTACTGROUPS;
			else if(!strcmp(variables[x],"services"))
				display_type=DISPLAY_SERVICES;
			else if(!strcmp(variables[x],"timeperiods"))
				display_type=DISPLAY_TIMEPERIODS;
			else if(!strcmp(variables[x],"commands"))
				display_type=DISPLAY_COMMANDS;
			else if(!strcmp(variables[x],"servicedependencies"))
				display_type=DISPLAY_SERVICEDEPENDENCIES;
			else if(!strcmp(variables[x],"serviceescalations"))
				display_type=DISPLAY_SERVICEESCALATIONS;
			else if(!strcmp(variables[x],"hostdependencies"))
				display_type=DISPLAY_HOSTDEPENDENCIES;
			else if(!strcmp(variables[x],"hostescalations"))
				display_type=DISPLAY_HOSTESCALATIONS;

			/* we found the embed option */
			else if(!strcmp(variables[x],"embedded"))
				embedded=TRUE;
		        }

		/* we received an invalid argument */
		else
			error=TRUE;
	
	        }

	/* free memory allocated to the CGI variables */
	free_cgivars(variables);

	return error;
        }



void display_hosts(void){
	host *temp_host=NULL;
	hostsmember *temp_hostsmember=NULL;
	contactsmember *temp_contactsmember=NULL;
	contactgroupsmember *temp_contactgroupsmember=NULL;
	char *processed_string=NULL;
	int options=0;
	int odd=0;
	char time_string[16];
	char *bg_class="";
	int contact=0;

	/* see if user is authorized to view host information... */
	if(is_authorized_for_configuration_information(&current_authdata)==FALSE){
		unauthorized_message();
		return;
	        }

	printf("<P><DIV ALIGN=CENTER CLASS='dataTitle'>ホスト</DIV></P>\n");

	printf("<P><DIV ALIGN=CENTER>\n");
	printf("<TABLE BORDER=0 CLASS='data'>\n");

	printf("<TR>\n");
	printf("<TH CLASS='data'>ホスト名</TH>");
	printf("<TH CLASS='data'>別名/説明</TH>");
	printf("<TH CLASS='data'>アドレス</TH>");
	printf("<TH CLASS='data'>親ホスト</TH>");
	printf("<TH CLASS='data'>最大チェック回数</TH>");
	printf("<TH CLASS='data'>チェック間隔</TH>\n");
	printf("<TH CLASS='data'>リトライ間隔</TH>\n");
	printf("<TH CLASS='data'>ホストチェックコマンド</TH>");
	printf("<TH CLASS='data'>Obsess Over</TH>\n");
	printf("<TH CLASS='data'>アクティブチェック</TH>\n");
	printf("<TH CLASS='data'>パッシブチェック</TH>\n");
	printf("<TH CLASS='data'>フレッシュチェック</TH>\n");
	printf("<TH CLASS='data'>フレッシュ閾値</TH>\n");
	printf("<TH CLASS='data'>デフォルトの通知先グループ</TH>\n");
	printf("<TH CLASS='data'>通知間隔</TH>");
	printf("<TH CLASS='data'>初期通知遅延</TH>");
	printf("<TH CLASS='data'>通知オプション</TH>");
	printf("<TH CLASS='data'>通知時間帯</TH>");
	printf("<TH CLASS='data'>イベントハンドラ</TH>");
	printf("<TH CLASS='data'>イベントハンドラの有効化</TH>");
	printf("<TH CLASS='data'>ストーキングオプション</TH>\n");
	printf("<TH CLASS='data'>フラップ検知設定</TH>");
	printf("<TH CLASS='data'>フラップ低閾値</TH>");
	printf("<TH CLASS='data'>フラップ高閾値</TH>");
	printf("<TH CLASS='data'>フラップ検知オプション</TH>\n");
	printf("<TH CLASS='data'>処理パフォーマンスデータ</TH>");
	printf("<TH CLASS='data'>予測機能</TH>");
	printf("<TH CLASS='data'>予測機能オプション</TH>");
	printf("<TH CLASS='data'>ノート</TH>");
	printf("<TH CLASS='data'>ノートURL</TH>");
	printf("<TH CLASS='data'>操作URL</TH>");
	printf("<TH CLASS='data'>2-D Coords</TH>");
	printf("<TH CLASS='data'>3-D Coords</TH>");
	printf("<TH CLASS='data'>ステータスマップ画像</TH>");
	printf("<TH CLASS='data'>VRML画像</TH>");
	printf("<TH CLASS='data'>ロゴ画像</TH>");
	printf("<TH CLASS='data'>画像Alt</TH>");
	printf("<TH CLASS='data'>ステータス保存オプション</TH>");
	printf("</TR>\n");

	/* check all the hosts... */
	for(temp_host=host_list;temp_host!=NULL;temp_host=temp_host->next){

		/* grab macros */
		grab_host_macros(temp_host);

		if(odd){
			odd=0;
			bg_class="dataOdd";
		        }
		else{
			odd=1;
			bg_class="dataEven";
		        }

		printf("<TR CLASS='%s'>\n",bg_class);

		printf("<TD CLASS='%s'><a name='%s'>%s</a></TD>\n",bg_class,url_encode(temp_host->name),temp_host->name);
		printf("<TD CLASS='%s'>%s</TD>\n",bg_class,temp_host->alias);
		printf("<TD CLASS='%s'>%s</TD>\n",bg_class,temp_host->address);

		printf("<TD CLASS='%s'>",bg_class);
		for(temp_hostsmember=temp_host->parent_hosts;temp_hostsmember!=NULL;temp_hostsmember=temp_hostsmember->next){

			if(temp_hostsmember!=temp_host->parent_hosts)
				printf(", ");

			printf("<a href='%s?type=hosts#%s'>%s</a>\n",CONFIG_CGI,url_encode(temp_hostsmember->host_name),temp_hostsmember->host_name);
		        }
		if(temp_host->parent_hosts==NULL)
			printf("&nbsp;");
		printf("</TD>\n");

		printf("<TD CLASS='%s'>%d</TD>\n",bg_class,temp_host->max_attempts);

		get_interval_time_string(temp_host->check_interval,time_string,sizeof(time_string));
		printf("<TD CLASS='%s'>%s</TD>\n",bg_class,time_string);

		get_interval_time_string(temp_host->retry_interval,time_string,sizeof(time_string));
		printf("<TD CLASS='%s'>%s</TD>\n",bg_class,time_string);

		printf("<TD CLASS='%s'>",bg_class);
		if(temp_host->host_check_command==NULL)
			printf("&nbsp;");
		else
			printf("<a href='%s?type=commands#%s'>%s</a></TD>\n",CONFIG_CGI,url_encode(temp_host->host_check_command),html_encode(temp_host->host_check_command,FALSE));
		printf("</TD>\n");

		printf("<TD CLASS='%s'>%s</TD>\n",bg_class,(temp_host->obsess_over_host==TRUE)?"有効":"無効");

		printf("<TD CLASS='%s'>%s</TD>\n",bg_class,(temp_host->checks_enabled==TRUE)?"有効":"無効");

		printf("<TD CLASS='%s'>%s</TD>\n",bg_class,(temp_host->accept_passive_host_checks==TRUE)?"有効":"無効");

		printf("<TD CLASS='%s'>%s</TD>\n",bg_class,(temp_host->check_freshness==TRUE)?"有効":"無効");

		printf("<TD CLASS='%s'>",bg_class);
		if(temp_host->freshness_threshold==0)
			printf("自動設定\n");
		else
			printf("%d秒\n",temp_host->freshness_threshold);
		printf("</TD>\n");

		printf("<TD CLASS='%s'>",bg_class);

		/* find all the contacts for this host... */
		contact=0;
		for(temp_contactsmember=temp_host->contacts;temp_contactsmember!=NULL;temp_contactsmember=temp_contactsmember->next){
			contact++;
			if(contact>1)
				printf(", ");

			printf("<A HREF='%s?type=contacts#%s'>%s</A>\n",CONFIG_CGI,url_encode(temp_contactsmember->contact_name),temp_contactsmember->contact_name);
		        }
		for(temp_contactgroupsmember=temp_host->contact_groups;temp_contactgroupsmember!=NULL;temp_contactgroupsmember=temp_contactgroupsmember->next){
			contact++;
			if(contact>1)
				printf(", ");
			printf("<A HREF='%s?type=contactgroups#%s'>%s</A>\n",CONFIG_CGI,url_encode(temp_contactgroupsmember->group_name),temp_contactgroupsmember->group_name);
			}
		if(contact==0)
			printf("&nbsp;");
		printf("</TD>\n");

		get_interval_time_string(temp_host->notification_interval,time_string,sizeof(time_string));
		printf("<TD CLASS='%s'>%s</TD>\n",bg_class,(temp_host->notification_interval==0)?"<i>再通知無し</I>":time_string);

		get_interval_time_string(temp_host->first_notification_delay,time_string,sizeof(time_string));
		printf("<TD CLASS='%s'>%s</TD>\n",bg_class,time_string);

		printf("<TD CLASS='%s'>",bg_class);
		options=0;
		if(temp_host->notify_on_down==TRUE){
			options=1;
			printf("停止");
		        }
		if(temp_host->notify_on_unreachable==TRUE){
			printf("%s未到達",(options)?", ":"");
			options=1;
		        }
		if(temp_host->notify_on_recovery==TRUE){
			printf("%s復旧",(options)?", ":"");
			options=1;
		        }
		if(temp_host->notify_on_flapping==TRUE){
			printf("%sフラッピング",(options)?", ":"");
			options=1;
		        }
		if(temp_host->notify_on_downtime==TRUE){
			printf("%sダウンタイム",(options)?", ":"");
			options=1;
		        }
		if(options==0)
			printf("無し");
		printf("</TD>\n");

		printf("<TD CLASS='%s'>",bg_class);
		if(temp_host->notification_period==NULL)
			printf("&nbsp;");
		else
			printf("<a href='%s?type=timeperiods#%s'>%s</a>",CONFIG_CGI,url_encode(temp_host->notification_period),temp_host->notification_period);
		printf("</TD>\n");

		printf("<TD CLASS='%s'>",bg_class);
		if(temp_host->event_handler==NULL)
			printf("&nbsp");
		else
			printf("<a href='%s?type=commands#%s'>%s</a></TD>\n",CONFIG_CGI,url_encode(temp_host->event_handler),temp_host->event_handler);
		printf("</TD>\n");

		printf("<TD CLASS='%s'>",bg_class);
		printf("%s\n",(temp_host->event_handler_enabled==TRUE)?"有効":"無効");
		printf("</TD>\n");

		printf("<TD CLASS='%s'>",bg_class);
		options=0;
		if(temp_host->stalk_on_up==TRUE){
			options=1;
			printf("稼動");
		        }
		if(temp_host->stalk_on_down==TRUE){
			printf("%s停止",(options)?", ":"");
			options=1;
		        }
		if(temp_host->stalk_on_unreachable==TRUE){
			printf("%s未到達",(options)?", ":"");
			options=1;
		        }
		if(options==0)
			printf("無し");
		printf("</TD>\n");

		printf("<TD CLASS='%s'>",bg_class);
		printf("%s\n",(temp_host->flap_detection_enabled==TRUE)?"有効":"無効");
		printf("</TD>\n");

		printf("<TD CLASS='%s'>",bg_class);
		if(temp_host->low_flap_threshold==0.0)
			printf("グローバル設定\n");
		else
			printf("%3.1f%%\n",temp_host->low_flap_threshold);
		printf("</TD>\n");

		printf("<TD CLASS='%s'>",bg_class);
		if(temp_host->high_flap_threshold==0.0)
			printf("グローバル設定\n");
		else
			printf("%3.1f%%\n",temp_host->high_flap_threshold);
		printf("</TD>\n");

		printf("<TD CLASS='%s'>",bg_class);
		options=0;
		if(temp_host->flap_detection_on_up==TRUE){
			options=1;
			printf("稼動");
		        }
		if(temp_host->flap_detection_on_down==TRUE){
			printf("%s停止",(options)?", ":"");
			options=1;
		        }
		if(temp_host->flap_detection_on_unreachable==TRUE){
			printf("%s未到達",(options)?", ":"");
			options=1;
		        }
		if(options==0)
			printf("無し");
		printf("</TD>\n");

		printf("<TD CLASS='%s'>",bg_class);
		printf("%s\n",(temp_host->process_performance_data==TRUE)?"有効":"無効");
		printf("</TD>\n");

		printf("<TD CLASS='%s'>",bg_class);
		printf("%s\n",(temp_host->failure_prediction_enabled==TRUE)?"有効":"無効");
		printf("</TD>\n");

		printf("<TD CLASS='%s'>%s</TD>\n",bg_class,(temp_host->failure_prediction_options==NULL)?"&nbsp;":temp_host->failure_prediction_options);

		printf("<TD CLASS='%s'>%s</TD>",bg_class,(temp_host->notes==NULL)?"&nbsp;":temp_host->notes);

		printf("<TD CLASS='%s'>%s</TD>",bg_class,(temp_host->notes_url==NULL)?"&nbsp;":temp_host->notes_url);

		printf("<TD CLASS='%s'>%s</TD>",bg_class,(temp_host->action_url==NULL)?"&nbsp;":temp_host->action_url);

		if(temp_host->have_2d_coords==FALSE)
			printf("<TD CLASS='%s'>&nbsp;</TD>",bg_class);
		else
			printf("<TD CLASS='%s'>%d,%d</TD>",bg_class,temp_host->x_2d,temp_host->y_2d);

		if(temp_host->have_3d_coords==FALSE)
			printf("<TD CLASS='%s'>&nbsp;</TD>",bg_class);
		else
			printf("<TD CLASS='%s'>%.2f,%.2f,%.2f</TD>",bg_class,temp_host->x_3d,temp_host->y_3d,temp_host->z_3d);

		if(temp_host->statusmap_image==NULL)
			printf("<TD CLASS='%s'>&nbsp;</TD>",bg_class);
		else
			printf("<TD CLASS='%s' valign='center'><img src='%s%s' border='0' width='20' height='20'> %s</TD>",bg_class,url_logo_images_path,temp_host->statusmap_image,temp_host->statusmap_image);

		if(temp_host->vrml_image==NULL)
			printf("<TD CLASS='%s'>&nbsp;</TD>",bg_class);
		else
			printf("<TD CLASS='%s' valign='center'><img src='%s%s' border='0' width='20' height='20'> %s</TD>",bg_class,url_logo_images_path,temp_host->vrml_image,temp_host->vrml_image);

		if(temp_host->icon_image==NULL)
			printf("<TD CLASS='%s'>&nbsp;</TD>",bg_class);
		else{
			process_macros(temp_host->icon_image,&processed_string,0);
			printf("<TD CLASS='%s' valign='center'><img src='%s%s' border='0' width='20' height='20'> %s</TD>",bg_class,url_logo_images_path,processed_string,temp_host->icon_image);
			free(processed_string);
			}

		printf("<TD CLASS='%s'>%s</TD>",bg_class,(temp_host->icon_image_alt==NULL)?"&nbsp;":temp_host->icon_image_alt);

		printf("<TD CLASS='%s'>",bg_class);
		options=0;
		if(temp_host->retain_status_information==TRUE){
			options=1;
			printf("ステータス情報");
		        }
		if(temp_host->retain_nonstatus_information==TRUE){
			printf("%s非ステータス情報",(options==1)?", ":"");
			options=1;
		        }
		if(options==0)
			printf("無し");
		printf("</TD>\n");

		printf("</TR>\n");
	        }

	printf("</TABLE>\n");
	printf("</DIV>\n");
	printf("</P>\n");

	return;
        }



void display_hostgroups(void){
	hostgroup *temp_hostgroup;
	hostsmember *temp_hostsmember;
	int odd=0;
	char *bg_class="";

	/* see if user is authorized to view hostgroup information... */
	if(is_authorized_for_configuration_information(&current_authdata)==FALSE){
		unauthorized_message();
		return;
	        }

	printf("<P><DIV ALIGN=CENTER CLASS='dataTitle'>ホストグループ</DIV></P>\n");

	printf("<P>\n");
	printf("<DIV ALIGN=CENTER>\n");

	printf("<TABLE BORDER=0 CLASS='data'>\n");
	printf("<TR>\n");
	printf("<TH CLASS='data'>グループ名</TH>");
	printf("<TH CLASS='data'>説明</TH>");
	printf("<TH CLASS='data'>このグループのメンバー</TH>");
	printf("<TH CLASS='data'>ノート</TH>");
	printf("<TH CLASS='data'>ノートURL</TH>");
	printf("<TH CLASS='data'>操作URL</TH>");
	printf("</TR>\n");

	/* check all the hostgroups... */
	for(temp_hostgroup=hostgroup_list;temp_hostgroup!=NULL;temp_hostgroup=temp_hostgroup->next){

		if(odd){
			odd=0;
			bg_class="dataOdd";
		        }
		else{
			odd=1;
			bg_class="dataEven";
		        }

		printf("<TR CLASS='%s'>\n",bg_class);

		printf("<TD CLASS='%s'>%s</TD>",bg_class,temp_hostgroup->group_name);

		printf("<TD CLASS='%s'>%s</TD>\n",bg_class,temp_hostgroup->alias);

		printf("<TD CLASS='%s'>",bg_class);

		/* find all the hosts that are members of this hostgroup... */
		for(temp_hostsmember=temp_hostgroup->members;temp_hostsmember!=NULL;temp_hostsmember=temp_hostsmember->next){

			if(temp_hostsmember!=temp_hostgroup->members)
				printf(", ");
			printf("<A HREF='%s?type=hosts#%s'>%s</A>\n",CONFIG_CGI,url_encode(temp_hostsmember->host_name),temp_hostsmember->host_name);
		        }
		printf("</TD>\n");

		printf("<TD CLASS='%s'>%s</TD>",bg_class,(temp_hostgroup->notes==NULL)?"&nbsp;":temp_hostgroup->notes);

		printf("<TD CLASS='%s'>%s</TD>",bg_class,(temp_hostgroup->notes_url==NULL)?"&nbsp;":temp_hostgroup->notes_url);

		printf("<TD CLASS='%s'>%s</TD>",bg_class,(temp_hostgroup->action_url==NULL)?"&nbsp;":temp_hostgroup->action_url);

		printf("</TR>\n");
	        }

	printf("</TABLE>\n");
	printf("</DIV>\n");
	printf("</P>\n");

	return;
        }



void display_servicegroups(void){
	servicegroup *temp_servicegroup;
	servicesmember *temp_servicesmember;
	int odd=0;
	char *bg_class="";

	/* see if user is authorized to view servicegroup information... */
	if(is_authorized_for_configuration_information(&current_authdata)==FALSE){
		unauthorized_message();
		return;
	        }

	printf("<P><DIV ALIGN=CENTER CLASS='dataTitle'>サービスグループ</DIV></P>\n");

	printf("<P>\n");
	printf("<DIV ALIGN=CENTER>\n");

	printf("<TABLE BORDER=0 CLASS='data'>\n");
	printf("<TR>\n");
	printf("<TH CLASS='data'>グループ名</TH>");
	printf("<TH CLASS='data'>説明</TH>");
	printf("<TH CLASS='data'>このグループのメンバー</TH>");
	printf("<TH CLASS='data'>ノート</TH>");
	printf("<TH CLASS='data'>ノートURL</TH>");
	printf("<TH CLASS='data'>操作URL</TH>");
	printf("</TR>\n");

	/* check all the servicegroups... */
	for(temp_servicegroup=servicegroup_list;temp_servicegroup!=NULL;temp_servicegroup=temp_servicegroup->next){

		if(odd){
			odd=0;
			bg_class="dataOdd";
		        }
		else{
			odd=1;
			bg_class="dataEven";
		        }

		printf("<TR CLASS='%s'>\n",bg_class);

		printf("<TD CLASS='%s'>%s</TD>",bg_class,temp_servicegroup->group_name);

		printf("<TD CLASS='%s'>%s</TD>\n",bg_class,temp_servicegroup->alias);

		printf("<TD CLASS='%s'>",bg_class);

		/* find all the services that are members of this servicegroup... */
		for(temp_servicesmember=temp_servicegroup->members;temp_servicesmember!=NULL;temp_servicesmember=temp_servicesmember->next){

			printf("%s<A HREF='%s?type=hosts#%s'>%s</A> / ",(temp_servicesmember==temp_servicegroup->members)?"":", ",CONFIG_CGI,url_encode(temp_servicesmember->host_name),temp_servicesmember->host_name);

			printf("<A HREF='%s?type=services#%s;",CONFIG_CGI,url_encode(temp_servicesmember->host_name));
			printf("%s'>%s</A>\n",url_encode(temp_servicesmember->service_description),temp_servicesmember->service_description);
		        }

		printf("</TD>\n");

		printf("<TD CLASS='%s'>%s</TD>",bg_class,(temp_servicegroup->notes==NULL)?"&nbsp;":temp_servicegroup->notes);

		printf("<TD CLASS='%s'>%s</TD>",bg_class,(temp_servicegroup->notes_url==NULL)?"&nbsp;":temp_servicegroup->notes_url);

		printf("<TD CLASS='%s'>%s</TD>",bg_class,(temp_servicegroup->action_url==NULL)?"&nbsp;":temp_servicegroup->action_url);

		printf("</TR>\n");
	        }

	printf("</TABLE>\n");
	printf("</DIV>\n");
	printf("</P>\n");

	return;
        }



void display_contacts(void){
	contact *temp_contact;
	commandsmember *temp_commandsmember;
	int odd=0;
	int options;
	int found;
	char *bg_class="";

	/* see if user is authorized to view contact information... */
	if(is_authorized_for_configuration_information(&current_authdata)==FALSE){
		unauthorized_message();
		return;
	        }

	printf("<P><DIV ALIGN=CENTER CLASS='dataTitle'>通知先</DIV></P>\n");

	printf("<P>\n");
	printf("<DIV ALIGN=CENTER>\n");

	printf("<TABLE CLASS='data'>\n");
    
	printf("<TR>\n");
	printf("<TH CLASS='data'>通知先名</TH>");
	printf("<TH CLASS='data'>別名</TH>");
	printf("<TH CLASS='data'>メールアドレス</TH>");
	printf("<TH CLASS='data'>携帯番号/アドレス</TH>");
	printf("<TH CLASS='data'>サービス通知オプション</TH>");
	printf("<TH CLASS='data'>ホスト通知オプション</TH>");
	printf("<TH CLASS='data'>サービス通知期間</TH>");
	printf("<TH CLASS='data'>ホスト通知期間</TH>");
	printf("<TH CLASS='data'>サービス通知コマンド</TH>");
	printf("<TH CLASS='data'>ホスト通知コマンド</TH>");
	printf("<TH CLASS='data'>保持オプション</TH>");
	printf("</TR>\n");
	
	/* check all contacts... */
	for(temp_contact=contact_list;temp_contact!=NULL;temp_contact=temp_contact->next){

		if(odd){
			odd=0;
			bg_class="dataOdd";
		        }
		else{
			odd=1;
			bg_class="dataEven";
		        }

		printf("<TR CLASS='%s'>\n",bg_class);

		printf("<TD CLASS='%s'><A NAME='%s'>%s</a></TD>\n",bg_class,url_encode(temp_contact->name),temp_contact->name);
		printf("<TD CLASS='%s'>%s</TD>\n",bg_class,temp_contact->alias);
		printf("<TD CLASS='%s'><A HREF='mailto:%s'>%s</A></TD>\n",bg_class,(temp_contact->email==NULL)?"&nbsp;":temp_contact->email,(temp_contact->email==NULL)?"&nbsp;":temp_contact->email);
		printf("<TD CLASS='%s'>%s</TD>\n",bg_class,(temp_contact->pager==NULL)?"&nbsp;":temp_contact->pager);

		printf("<TD CLASS='%s'>",bg_class);
		options=0;
		if(temp_contact->notify_on_service_unknown==TRUE){
			options=1;
			printf("不明");
		        }
		if(temp_contact->notify_on_service_warning==TRUE){
			printf("%s警告",(options)?", ":"");
			options=1;
		        }
		if(temp_contact->notify_on_service_critical==TRUE){
			printf("%s障害",(options)?", ":"");
			options=1;
		        }
		if(temp_contact->notify_on_service_recovery==TRUE){
			printf("%s復旧",(options)?", ":"");
			options=1;
		        }
		if(temp_contact->notify_on_service_flapping==TRUE){
			printf("%sフラッピング",(options)?", ":"");
			options=1;
		        }
		if(temp_contact->notify_on_service_downtime==TRUE){
			printf("%sダウンタイム",(options)?", ":"");
			options=1;
		        }
		if(!options)
			printf("無し");
		printf("</TD>\n");

		printf("<TD CLASS='%s'>",bg_class);
		options=0;
		if(temp_contact->notify_on_host_down==TRUE){
			options=1;
			printf("停止");
		        }
		if(temp_contact->notify_on_host_unreachable==TRUE){
			printf("%s未到達",(options)?", ":"");
			options=1;
		        }
		if(temp_contact->notify_on_host_recovery==TRUE){
			printf("%s復旧",(options)?", ":"");
			options=1;
		        }
		if(temp_contact->notify_on_host_flapping==TRUE){
			printf("%sフラッピング",(options)?", ":"");
			options=1;
		        }
		if(temp_contact->notify_on_host_downtime==TRUE){
			printf("%sダウンタイム",(options)?", ":"");
			options=1;
		        }
		if(!options)
			printf("無し");
		printf("</TD>\n");

		printf("<TD CLASS='%s'>\n",bg_class);
		if(temp_contact->service_notification_period==NULL)
			printf("&nbsp;");
		else
			printf("<A HREF='%s?type=timeperiods#%s'>%s</A>",CONFIG_CGI,url_encode(temp_contact->service_notification_period),temp_contact->service_notification_period);
		printf("</TD>\n");

		printf("<TD CLASS='%s'>\n",bg_class);
		if(temp_contact->host_notification_period==NULL)
			printf("&nbsp;");
		else
			printf("<A HREF='%s?type=timeperiods#%s'>%s</A>",CONFIG_CGI,url_encode(temp_contact->host_notification_period),temp_contact->host_notification_period);
		printf("</TD>\n");

		printf("<TD CLASS='%s'>",bg_class);
		found=FALSE;
		for(temp_commandsmember=temp_contact->service_notification_commands;temp_commandsmember!=NULL;temp_commandsmember=temp_commandsmember->next){

			if(temp_commandsmember!=temp_contact->service_notification_commands)
				printf(", ");

			printf("<A HREF='%s?type=commands#%s'>%s</A>",CONFIG_CGI,url_encode(temp_commandsmember->command),temp_commandsmember->command);

			found=TRUE;
		        }
		if(found==FALSE)
			printf("無し");
		printf("</TD>\n");

		printf("<TD CLASS='%s'>",bg_class);
		found=FALSE;
		for(temp_commandsmember=temp_contact->host_notification_commands;temp_commandsmember!=NULL;temp_commandsmember=temp_commandsmember->next){

			if(temp_commandsmember!=temp_contact->host_notification_commands)
				printf(", ");

			printf("<A HREF='%s?type=commands#%s'>%s</A>",CONFIG_CGI,url_encode(temp_commandsmember->command),temp_commandsmember->command);

			found=TRUE;
		        }
		if(found==FALSE)
			printf("無し");
		printf("</TD>\n");

		printf("<TD CLASS='%s'>",bg_class);
		options=0;
		if(temp_contact->retain_status_information==TRUE){
			options=1;
			printf("ステータス情報");
		        }
		if(temp_contact->retain_nonstatus_information==TRUE){
			printf("%sNon-Status Information",(options==1)?", ":"");
			options=1;
		        }
		if(options==0)
			printf("無し");
		printf("</TD>\n");

		printf("</TR>\n");
	        }

	printf("</TABLE>\n");
	printf("</DIV>\n");
	printf("</P>\n");

	return;
        }



void display_contactgroups(void){
	contactgroup *temp_contactgroup;
	contactsmember *temp_contactsmember;
	int odd=0;
	char *bg_class="";

	/* see if user is authorized to view contactgroup information... */
	if(is_authorized_for_configuration_information(&current_authdata)==FALSE){
		unauthorized_message();
		return;
	        }


	printf("<P><DIV ALIGN=CENTER CLASS='dataTitle'>通知先グループ</DIV></P>\n");

	printf("<P>\n");
	printf("<DIV ALIGN=CENTER>\n");

	printf("<TABLE BORDER=0 CELLSPACING=3 CELLPADDING=0>\n");

	printf("<TR>\n");
	printf("<TH CLASS='data'>グループ名</TH>\n");
	printf("<TH CLASS='data'>説明</TH>\n");
	printf("<TH CLASS='data'>通知先メンバー</TH>\n");
	printf("</TR>\n");

	/* check all the contact groups... */
	for(temp_contactgroup=contactgroup_list;temp_contactgroup!=NULL;temp_contactgroup=temp_contactgroup->next){

		if(odd){
			odd=0;
			bg_class="dataOdd";
		        }
		else{
			odd=1;
			bg_class="dataEven";
		        }

		printf("<TR CLASS='%s'>\n",bg_class);

		printf("<TD CLASS='%s'><A NAME='%s'></A>%s</TD>\n",bg_class,url_encode(temp_contactgroup->group_name),temp_contactgroup->group_name);
		printf("<TD CLASS='%s'>%s</TD>\n",bg_class,temp_contactgroup->alias);

		/* find all the contact who are members of this contact group... */
		printf("<TD CLASS='%s'>",bg_class);
		for(temp_contactsmember=temp_contactgroup->members;temp_contactsmember!=NULL;temp_contactsmember=temp_contactsmember->next){
			
			if(temp_contactsmember!=temp_contactgroup->members)
				printf(", ");

			printf("<A HREF='%s?type=contacts#%s'>%s</A>\n",CONFIG_CGI,url_encode(temp_contactsmember->contact_name),temp_contactsmember->contact_name);
		        }
		printf("</TD>\n");

		printf("</TR>\n");
	        }

	printf("</TABLE>\n");
	printf("</DIV>\n");
	printf("</P>\n");

	return;
        }



void display_services(void){
	service *temp_service=NULL;
	contactsmember *temp_contactsmember=NULL;
	contactgroupsmember *temp_contactgroupsmember=NULL;
	char *processed_string=NULL;
	char command_line[MAX_INPUT_BUFFER];
	char *command_name="";
	int options;
	int odd=0;
	char time_string[16];
	char *bg_class;
	int contact=0;


	/* see if user is authorized to view service information... */
	if(is_authorized_for_configuration_information(&current_authdata)==FALSE){
		unauthorized_message();
		return;
	        }

	printf("<P><DIV ALIGN=CENTER CLASS='dataTitle'>サービス</DIV></P>\n");

	printf("<P>\n");
	printf("<DIV ALIGN=CENTER>\n");

	printf("<TABLE BORDER=0 CLASS='data'>\n");
	printf("<TR>\n");
	printf("<TH CLASS='data' COLSPAN=2>サービス</TH>");
	printf("</TR>\n");
	printf("<TR>\n");
	printf("<TH CLASS='data'>ホスト</TH>\n");
	printf("<TH CLASS='data'>説明</TH>\n");
	printf("<TH CLASS='data'>最大チェック回数</TH>\n");
	printf("<TH CLASS='data'>通常のチェック間隔</TH>\n");
	printf("<TH CLASS='data'>再試行のチェック間隔</TH>\n");
	printf("<TH CLASS='data'>チェックコマンド</TH>\n");
	printf("<TH CLASS='data'>チェック時間帯</TH>\n");
	printf("<TH CLASS='data'>並列処理</TH>\n");
	printf("<TH CLASS='data'>Volatile</TH>\n");
	printf("<TH CLASS='data'>Obsess Over</TH>\n");
	printf("<TH CLASS='data'>アクティブチェック</TH>\n");
	printf("<TH CLASS='data'>パッシブチェック</TH>\n");
	printf("<TH CLASS='data'>フレッシュチェック</TH>\n");
	printf("<TH CLASS='data'>フレッシュ閾値</TH>\n");
	printf("<TH CLASS='data'>デフォルトの通知先グループ</TH>\n");
	printf("<TH CLASS='data'>通知</TH>\n");
	printf("<TH CLASS='data'>通知間隔</TH>\n");
	printf("<TH CLASS='data'>初期通知遅延</TH>\n");
	printf("<TH CLASS='data'>通知オプション</TH>\n");
	printf("<TH CLASS='data'>通知時間帯</TH>\n");
	printf("<TH CLASS='data'>イベントハンドラ</TH>");
	printf("<TH CLASS='data'>イベントハンドラの有効化</TH>");
	printf("<TH CLASS='data'>ストーキングオプション</TH>\n");
	printf("<TH CLASS='data'>フラップ検知設定</TH>");
	printf("<TH CLASS='data'>フラップ低閾値</TH>");
	printf("<TH CLASS='data'>フラップ高閾値</TH>");
	printf("<TH CLASS='data'>フラップ検知オプション</TH>");
	printf("<TH CLASS='data'>処理パフォーマンスデータ</TH>");
	printf("<TH CLASS='data'>予測機能</TH>");
	printf("<TH CLASS='data'>予測機能オプション</TH>");
	printf("<TH CLASS='data'>ノート</TH>");
	printf("<TH CLASS='data'>ノートURL</TH>");
	printf("<TH CLASS='data'>操作URL</TH>");
	printf("<TH CLASS='data'>ロゴ画像</TH>");
	printf("<TH CLASS='data'>画像Alt</TH>");
	printf("<TH CLASS='data'>保持オプション</TH>");
	printf("</TR>\n");

	/* check all the services... */
	for(temp_service=service_list;temp_service!=NULL;temp_service=temp_service->next){

		/* grab macros */
		grab_service_macros(temp_service);

		if(odd){
			odd=0;
			bg_class="dataOdd";
	                }
		else{
			odd=1;
			bg_class="dataEven";
	                }

		printf("<TR CLASS='%s'>\n",bg_class);

		printf("<TD CLASS='%s'><A NAME='%s;",bg_class,url_encode(temp_service->host_name));
		printf("%s'></A>",url_encode(temp_service->description));
		printf("<A HREF='%s?type=hosts#%s'>%s</A></TD>\n",CONFIG_CGI,url_encode(temp_service->host_name),temp_service->host_name);
		
		printf("<TD CLASS='%s'>%s</TD>\n",bg_class,temp_service->description);
		
		printf("<TD CLASS='%s'>%d</TD>\n",bg_class,temp_service->max_attempts);

		get_interval_time_string(temp_service->check_interval,time_string,sizeof(time_string));
		printf("<TD CLASS='%s'>%s</TD>\n",bg_class,time_string);
		get_interval_time_string(temp_service->retry_interval,time_string,sizeof(time_string));
		printf("<TD CLASS='%s'>%s</TD>\n",bg_class,time_string);

		strncpy(command_line,temp_service->service_check_command,sizeof(command_line));
		command_line[sizeof(command_line)-1]='\x0';
		command_name=strtok(command_line,"!");

		printf("<TD CLASS='%s'><A HREF='%s?type=commands#%s'>%s</A></TD>\n",bg_class,CONFIG_CGI,url_encode(command_name),html_encode(temp_service->service_check_command,FALSE));
		printf("<TD CLASS='%s'>",bg_class);
		if(temp_service->check_period==NULL)
			printf("&nbsp;");
		else
			printf("<A HREF='%s?type=timeperiods#%s'>%s</A>",CONFIG_CGI,url_encode(temp_service->check_period),temp_service->check_period);
		printf("</TD>\n");

		printf("<TD CLASS='%s'>%s</TD>\n",bg_class,(temp_service->parallelize==TRUE)?"有効":"無効");

		printf("<TD CLASS='%s'>%s</TD>\n",bg_class,(temp_service->is_volatile==TRUE)?"有効:":"無効");

		printf("<TD CLASS='%s'>%s</TD>\n",bg_class,(temp_service->obsess_over_service==TRUE)?"有効:":"無効");

		printf("<TD CLASS='%s'>%s</TD>\n",bg_class,(temp_service->checks_enabled==TRUE)?"有効:":"無効");

		printf("<TD CLASS='%s'>%s</TD>\n",bg_class,(temp_service->accept_passive_service_checks==TRUE)?"有効:":"無効");

		printf("<TD CLASS='%s'>%s</TD>\n",bg_class,(temp_service->check_freshness==TRUE)?"有効:":"無効");

		printf("<TD CLASS='%s'>",bg_class);
		if(temp_service->freshness_threshold==0)
			printf("自動設定\n");
		else
			printf("%d秒\n",temp_service->freshness_threshold);
		printf("</TD>\n");

		printf("<TD CLASS='%s'>",bg_class);
		contact=0;
		for(temp_contactsmember=temp_service->contacts;temp_contactsmember!=NULL;temp_contactsmember=temp_contactsmember->next){
			contact++;
			if(contact>1)
				printf(", ");
			printf("<A HREF='%s?type=contacts#%s'>%s</A>",CONFIG_CGI,url_encode(temp_contactsmember->contact_name),temp_contactsmember->contact_name);
	                }
		for(temp_contactgroupsmember=temp_service->contact_groups;temp_contactgroupsmember!=NULL;temp_contactgroupsmember=temp_contactgroupsmember->next){
			contact++;
			if(contact>1)
				printf(", ");
			printf("<A HREF='%s?type=contactgroups#%s'>%s</A>\n",CONFIG_CGI,url_encode(temp_contactgroupsmember->group_name),temp_contactgroupsmember->group_name);
			}
		if(contact==0)
			printf("&nbsp;");
		printf("</TD>\n");

		printf("<TD CLASS='%s'>",bg_class);
		printf("%s\n",(temp_service->notifications_enabled==TRUE)?"有効:":"無効");
		printf("</TD>\n");

		get_interval_time_string(temp_service->notification_interval,time_string,sizeof(time_string));
		printf("<TD CLASS='%s'>%s</TD>\n",bg_class,(temp_service->notification_interval==0)?"<i>No Re-notification</i>":time_string);

		get_interval_time_string(temp_service->first_notification_delay,time_string,sizeof(time_string));
		printf("<TD CLASS='%s'>%s</TD>\n",bg_class,time_string);

		printf("<TD CLASS='%s'>",bg_class);
		options=0;
		if(temp_service->notify_on_unknown==TRUE){
			options=1;
			printf("不明");
	                }
		if(temp_service->notify_on_warning==TRUE){
			printf("%s警告",(options)?", ":"");
			options=1;
	                }
		if(temp_service->notify_on_critical==TRUE){
			printf("%s障害",(options)?", ":"");
			options=1;
	                }
		if(temp_service->notify_on_recovery==TRUE){
			printf("%s復旧",(options)?", ":"");
			options=1;
	                }
		if(temp_service->notify_on_flapping==TRUE){
			printf("%sフラッピング",(options)?", ":"");
			options=1;
	                }
		if(temp_service->notify_on_downtime==TRUE){
			printf("%sダウンタイム",(options)?", ":"");
			options=1;
	                }
		if(!options)
			printf("無し");
		printf("</TD>\n");
		printf("<TD CLASS='%s'>",bg_class);
		if(temp_service->notification_period==NULL)
			printf("&nbsp;");
		else
			printf("<A HREF='%s?type=timeperiods#%s'>%s</A>",CONFIG_CGI,url_encode(temp_service->notification_period),temp_service->notification_period);
		printf("</TD>\n");
		printf("<TD CLASS='%s'>",bg_class);
		if(temp_service->event_handler==NULL)
			printf("&nbsp;");
		else
			printf("<A HREF='%s?type=commands#%s'>%s</A>",CONFIG_CGI,url_encode(temp_service->event_handler),temp_service->event_handler);
		printf("</TD>\n");

		printf("<TD CLASS='%s'>",bg_class);
		printf("%s\n",(temp_service->event_handler_enabled==TRUE)?"Yes":"No");
		printf("</TD>\n");

		printf("<TD CLASS='%s'>",bg_class);
		options=0;
		if(temp_service->stalk_on_ok==TRUE){
			options=1;
			printf("正常");
	                }
		if(temp_service->stalk_on_warning==TRUE){
			printf("%s警告",(options)?", ":"");
			options=1;
	                }
		if(temp_service->stalk_on_unknown==TRUE){
			printf("%s不明",(options)?", ":"");
			options=1;
	                }
		if(temp_service->stalk_on_critical==TRUE){
			printf("%s障害",(options)?", ":"");
			options=1;
	                }
		if(options==0)
			printf("無し");
		printf("</TD>\n");

		printf("<TD CLASS='%s'>",bg_class);
		printf("%s\n",(temp_service->flap_detection_enabled==TRUE)?"有効":"無効");
		printf("</TD>\n");

		printf("<TD CLASS='%s'>",bg_class);
		if(temp_service->low_flap_threshold==0.0)
			printf("グローバル設定\n");
		else
			printf("%3.1f%%\n",temp_service->low_flap_threshold);
		printf("</TD>\n");
			
		printf("<TD CLASS='%s'>",bg_class);
		if(temp_service->high_flap_threshold==0.0)
			printf("グローバル設定\n");
		else
			printf("%3.1f%%\n",temp_service->high_flap_threshold);
		printf("</TD>\n");

		printf("<TD CLASS='%s'>",bg_class);
		options=0;
		if(temp_service->flap_detection_on_ok==TRUE){
			options=1;
			printf("正常");
	                }
		if(temp_service->flap_detection_on_warning==TRUE){
			printf("%s警告",(options)?", ":"");
			options=1;
	                }
		if(temp_service->flap_detection_on_unknown==TRUE){
			printf("%s不明",(options)?", ":"");
			options=1;
	                }
		if(temp_service->flap_detection_on_critical==TRUE){
			printf("%s障害",(options)?", ":"");
			options=1;
	                }
		if(options==0)
			printf("無し");
		printf("</TD>\n");

		printf("<TD CLASS='%s'>",bg_class);
		printf("%s\n",(temp_service->process_performance_data==TRUE)?"有効:":"無効");
		printf("</TD>\n");

		printf("<TD CLASS='%s'>",bg_class);
		printf("%s\n",(temp_service->failure_prediction_enabled==TRUE)?"有効:":"無効");
		printf("</TD>\n");

		printf("<TD CLASS='%s'>%s</TD>\n",bg_class,(temp_service->failure_prediction_options==NULL)?"&nbsp;":temp_service->failure_prediction_options);

		printf("<TD CLASS='%s'>%s</TD>",bg_class,(temp_service->notes==NULL)?"&nbsp;":temp_service->notes);

		printf("<TD CLASS='%s'>%s</TD>",bg_class,(temp_service->notes_url==NULL)?"&nbsp;":temp_service->notes_url);

		printf("<TD CLASS='%s'>%s</TD>",bg_class,(temp_service->action_url==NULL)?"&nbsp;":temp_service->action_url);

		if(temp_service->icon_image==NULL)
			printf("<TD CLASS='%s'>&nbsp;</TD>",bg_class);
		else{
			process_macros(temp_service->icon_image,&processed_string,0);
			printf("<TD CLASS='%s' valign='center'><img src='%s%s' border='0' width='20' height='20'> %s</TD>",bg_class,url_logo_images_path,processed_string,temp_service->icon_image);
			free(processed_string);
			}

		printf("<TD CLASS='%s'>%s</TD>",bg_class,(temp_service->icon_image_alt==NULL)?"&nbsp;":temp_service->icon_image_alt);

		printf("<TD CLASS='%s'>",bg_class);
		options=0;
		if(temp_service->retain_status_information==TRUE){
			options=1;
			printf("ステータス情報");
	                }
		if(temp_service->retain_nonstatus_information==TRUE){
			printf("%s非ステータス情報",(options==1)?", ":"");
			options=1;
	                }
		if(options==0)
			printf("無し");
		printf("</TD>\n");
		
		printf("</TR>\n");
	        }

	printf("</TABLE>\n");
	printf("</DIV>\n");
	printf("</P>\n");

	return;
        }



void display_timeperiods(void){
	timerange *temp_timerange=NULL;
	daterange *temp_daterange=NULL;
	timeperiod *temp_timeperiod=NULL;
	timeperiodexclusion *temp_timeperiodexclusion=NULL;
	/* char *months[12]={"january","february","march","april","may","june","july","august","september","october","november","december"}; */
	/* char *days[7]={"sunday","monday","tuesday","wednesday","thursday","friday","saturday"}; */
	char *months[12]={"1月","2月","3月","4月","5月","6月","7月","8月","9月","10月","11月","12月"};
	char *days[7]={"日","月","火","水","木","金","土"};
	int odd=0;
	int day=0;
	int x=0;
	char *bg_class="";
	char timestring[10];
	int hours=0;
	int minutes=0;
	int seconds=0;
	int line=0;
	int item=0;

	/* see if user is authorized to view time period information... */
	if(is_authorized_for_configuration_information(&current_authdata)==FALSE){
		unauthorized_message();
		return;
	        }

	printf("<P><DIV ALIGN=CENTER CLASS='dataTitle'>時間帯</DIV></P>\n");

	printf("<P>\n");
	printf("<DIV ALIGN=CENTER>\n");

	printf("<TABLE BORDER=0 CLASS='data'>\n");
	printf("<TR>\n");
	printf("<TH CLASS='data'>名前</TH>\n");
	printf("<TH CLASS='data'>別名/説明</TH>\n");
	printf("<TH CLASS='data'>除外</TH>\n");
	printf("<TH CLASS='data'>日付</TH>\n");
	printf("<TH CLASS='data'>時刻</TH>\n");
	printf("</TR>\n");

	/* check all the time periods... */
	for(temp_timeperiod=timeperiod_list;temp_timeperiod!=NULL;temp_timeperiod=temp_timeperiod->next){

		if(odd){
			odd=0;
			bg_class="dataOdd";
		        }
		else{
			odd=1;
			bg_class="dataEven";
		        }

		printf("<TR CLASS='%s'>\n",bg_class);

		printf("<TD CLASS='%s'><A NAME='%s'>%s</A></TD>\n",bg_class,url_encode(temp_timeperiod->name),temp_timeperiod->name);
		printf("<TD CLASS='%s'>%s</TD>\n",bg_class,temp_timeperiod->alias);

		printf("<TD CLASS='%s'>",bg_class);
		item=0;
		for(temp_timeperiodexclusion=temp_timeperiod->exclusions;temp_timeperiodexclusion!=NULL;temp_timeperiodexclusion=temp_timeperiodexclusion->next){
			item++;
			printf("%s<A HREF='#%s'>%s</A>",(item==1)?"":",&nbsp;",url_encode(temp_timeperiodexclusion->timeperiod_name),temp_timeperiodexclusion->timeperiod_name);
			}
		printf("</TD>");

		printf("<TD CLASS='%s'>",bg_class);

		line=0;
		for(x=0;x<DATERANGE_TYPES;x++){
			for(temp_daterange=temp_timeperiod->exceptions[x];temp_daterange!=NULL;temp_daterange=temp_daterange->next){

				line++;

				if(line>1)
					printf("<TR><TD COLSPAN='3'></TD><TD CLASS='%s'>\n",bg_class);

				switch(temp_daterange->type){
				case DATERANGE_CALENDAR_DATE:
					printf("%d-%02d-%02d",temp_daterange->syear,temp_daterange->smon,temp_daterange->smday);
					if((temp_daterange->smday!=temp_daterange->emday) || (temp_daterange->smon!=temp_daterange->emon) || (temp_daterange->syear!=temp_daterange->eyear))
						printf(" - %d-%02d-%02d",temp_daterange->eyear,temp_daterange->emon,temp_daterange->emday);
					if(temp_daterange->skip_interval>1)
						printf(" / %d",temp_daterange->skip_interval);
					break;
				case DATERANGE_MONTH_DATE:
					printf("%s %d",months[temp_daterange->smon],temp_daterange->smday);
					if((temp_daterange->smon!=temp_daterange->emon) || (temp_daterange->smday!=temp_daterange->emday)){
						printf(" - %s %d",months[temp_daterange->emon],temp_daterange->emday);
						if(temp_daterange->skip_interval>1)
							printf(" / %d",temp_daterange->skip_interval);
						}
					break;
				case DATERANGE_MONTH_DAY:
					printf("day %d",temp_daterange->smday);
					if(temp_daterange->smday!=temp_daterange->emday){
						printf(" - %d",temp_daterange->emday);
						if(temp_daterange->skip_interval>1)
							printf(" / %d",temp_daterange->skip_interval);
						}
					break;
				case DATERANGE_MONTH_WEEK_DAY:
					printf("%s %d %s",days[temp_daterange->swday],temp_daterange->swday_offset,months[temp_daterange->smon]);
					if((temp_daterange->smon!=temp_daterange->emon) || (temp_daterange->swday!=temp_daterange->ewday) || (temp_daterange->swday_offset!=temp_daterange->ewday_offset)){
						printf(" - %s %d %s",days[temp_daterange->ewday],temp_daterange->ewday_offset,months[temp_daterange->emon]);
						if(temp_daterange->skip_interval>1)
							printf(" / %d",temp_daterange->skip_interval);
						}
					break;
				case DATERANGE_WEEK_DAY:
					printf("%s %d",days[temp_daterange->swday],temp_daterange->swday_offset);
					if((temp_daterange->swday!=temp_daterange->ewday) || (temp_daterange->swday_offset!=temp_daterange->ewday_offset)){
						printf(" - %s %d",days[temp_daterange->ewday],temp_daterange->ewday_offset);
						if(temp_daterange->skip_interval>1)
							printf(" / %d",temp_daterange->skip_interval);
						}
					break;
				default:
					break;
					}

				printf("</TD><TD CLASS='%s'>\n",bg_class);

				for(temp_timerange=temp_daterange->times;temp_timerange!=NULL;temp_timerange=temp_timerange->next){

					if(temp_timerange!=temp_daterange->times)
						printf(", ");

					hours=temp_timerange->range_start/3600;
					minutes=(temp_timerange->range_start-(hours*3600))/60;
					seconds=temp_timerange->range_start-(hours*3600)-(minutes*60);
					snprintf(timestring,sizeof(timestring)-1,"%02d:%02d:%02d",hours,minutes,seconds);
					timestring[sizeof(timestring)-1]='\x0';
					printf("%s - ",timestring);

					hours=temp_timerange->range_end/3600;
					minutes=(temp_timerange->range_end-(hours*3600))/60;
					seconds=temp_timerange->range_end-(hours*3600)-(minutes*60);
					snprintf(timestring,sizeof(timestring)-1,"%02d:%02d:%02d",hours,minutes,seconds);
					timestring[sizeof(timestring)-1]='\x0';
					printf("%s",timestring);
					}

				printf("</TD>\n");
				printf("</TR>\n");
				}
			}
		for(day=0;day<7;day++){

			if(temp_timeperiod->days[day]==NULL)
				continue;
			
			line++;

			if(line>1)
				printf("<TR><TD COLSPAN='3'></TD><TD CLASS='%s'>\n",bg_class);

			printf("%s",days[day]);

			printf("</TD><TD CLASS='%s'>\n",bg_class);
				
			for(temp_timerange=temp_timeperiod->days[day];temp_timerange!=NULL;temp_timerange=temp_timerange->next){

				if(temp_timerange!=temp_timeperiod->days[day])
					printf(", ");

				hours=temp_timerange->range_start/3600;
				minutes=(temp_timerange->range_start-(hours*3600))/60;
				seconds=temp_timerange->range_start-(hours*3600)-(minutes*60);
				snprintf(timestring,sizeof(timestring)-1,"%02d:%02d:%02d",hours,minutes,seconds);
				timestring[sizeof(timestring)-1]='\x0';
				printf("%s - ",timestring);

				hours=temp_timerange->range_end/3600;
				minutes=(temp_timerange->range_end-(hours*3600))/60;
				seconds=temp_timerange->range_end-(hours*3600)-(minutes*60);
				snprintf(timestring,sizeof(timestring)-1,"%02d:%02d:%02d",hours,minutes,seconds);
				timestring[sizeof(timestring)-1]='\x0';
				printf("%s",timestring);
			        }

			printf("&nbsp;</TD>\n");
			printf("</TR>\n");
		        }

		if(line==0){
			printf("</TD>\n");
			printf("</TR>\n");
			}
	        }

	printf("</TABLE>\n");
	printf("</DIV>\n");
	printf("</P>\n");

	return;
        }



void display_commands(void){
	command *temp_command;
	int odd=0;
	char *bg_class="";

	/* see if user is authorized to view command information... */
	if(is_authorized_for_configuration_information(&current_authdata)==FALSE){
		unauthorized_message();
		return;
	        }

	printf("<P><DIV ALIGN=CENTER CLASS='dataTitle'>コマンド</DIV></P>\n");

	printf("<P><DIV ALIGN=CENTER>\n");
	printf("<TABLE BORDER=0 CLASS='data'>\n");
	printf("<TR><TH CLASS='data'>コマンド名</TH><TH CLASS='data'>コマンドライン</TH></TR>\n");

	/* check all commands */
	for(temp_command=command_list;temp_command!=NULL;temp_command=temp_command->next){

		if(odd){
			odd=0;
			bg_class="dataEven";
		        }
		else{
			odd=1;
			bg_class="dataOdd";
		        }

		printf("<TR CLASS='%s'>\n",bg_class);

		printf("<TD CLASS='%s'><A NAME='%s'></A>%s</TD>\n",bg_class,url_encode(temp_command->name),temp_command->name);
		printf("<TD CLASS='%s'>%s</TD>\n",bg_class,html_encode(temp_command->command_line,FALSE));

		printf("</TR>\n");
	        }

	printf("</TABLE>\n");
	printf("</DIV></P>\n");

	return;
        }



void display_servicedependencies(void){
	servicedependency *temp_sd;
	int odd=0;
	int options;
	char *bg_class="";

	/* see if user is authorized to view hostgroup information... */
	if(is_authorized_for_configuration_information(&current_authdata)==FALSE){
		unauthorized_message();
		return;
	        }

	printf("<P><DIV ALIGN=CENTER CLASS='dataTitle'>サービス依存関係</DIV></P>\n");

	printf("<P>\n");
	printf("<DIV ALIGN=CENTER>\n");

	printf("<TABLE BORDER=0 CLASS='data'>\n");
	printf("<TR>\n");
	printf("<TH CLASS='data' COLSPAN=2>依存サービス</TH>");
	printf("<TH CLASS='data' COLSPAN=2>親サービス</TH>");
	printf("</TR>\n");
	printf("<TR>\n");
	printf("<TH CLASS='data'>ホスト</TH>");
	printf("<TH CLASS='data'>サービス</TH>");
	printf("<TH CLASS='data'>ホスト</TH>");
	printf("<TH CLASS='data'>サービス</TH>");
	printf("<TH CLASS='data'>依存タイプ</TH>");
	printf("<TH CLASS='data'>依存期間</TH>");
	printf("<TH CLASS='data'>依存失敗オプション</TH>");
	printf("</TR>\n");

	/* check all the service dependencies... */
	for(temp_sd=servicedependency_list;temp_sd!=NULL;temp_sd=temp_sd->next){

		if(odd){
			odd=0;
			bg_class="dataOdd";
		        }
		else{
			odd=1;
			bg_class="dataEven";
		        }

		printf("<TR CLASS='%s'>\n",bg_class);

		printf("<TD CLASS='%s'><A HREF='%s?type=hosts#%s'>%s</A></TD>",bg_class,CONFIG_CGI,url_encode(temp_sd->dependent_host_name),temp_sd->dependent_host_name);

		printf("<TD CLASS='%s'><A HREF='%s?type=services#%s;",bg_class,CONFIG_CGI,url_encode(temp_sd->dependent_host_name));
		printf("%s'>%s</A></TD>\n",url_encode(temp_sd->dependent_service_description),temp_sd->dependent_service_description);

		printf("<TD CLASS='%s'><A HREF='%s?type=hosts#%s'>%s</A></TD>",bg_class,CONFIG_CGI,url_encode(temp_sd->host_name),temp_sd->host_name);

		printf("<TD CLASS='%s'><A HREF='%s?type=services#%s;",bg_class,CONFIG_CGI,url_encode(temp_sd->host_name));
		printf("%s'>%s</A></TD>\n",url_encode(temp_sd->service_description),temp_sd->service_description);

		printf("<TD CLASS='%s'>%s</TD>",bg_class,(temp_sd->dependency_type==NOTIFICATION_DEPENDENCY)?"通知":"チェック実行");

		printf("<TD CLASS='%s'>",bg_class);
		if(temp_sd->dependency_period==NULL)
			printf("&nbsp;");
		else
			printf("<A HREF='%s?type=timeperiods#%s'>%s</A>",CONFIG_CGI,url_encode(temp_sd->dependency_period),temp_sd->dependency_period);
		printf("</TD>\n");

		printf("<TD CLASS='%s'>",bg_class);
		options=FALSE;
		if(temp_sd->fail_on_ok==TRUE){
			printf("正常");
			options=TRUE;
		        }
		if(temp_sd->fail_on_warning==TRUE){
			printf("%s警告",(options==TRUE)?", ":"");
			options=TRUE;
		        }
		if(temp_sd->fail_on_unknown==TRUE){
			printf("%s不明",(options==TRUE)?", ":"");
			options=TRUE;
		        }
		if(temp_sd->fail_on_critical==TRUE){
			printf("%s障害",(options==TRUE)?", ":"");
			options=TRUE;
		        }
		if(temp_sd->fail_on_pending==TRUE){
			printf("%s未解決",(options==TRUE)?", ":"");
			options=TRUE;
		        }
		printf("</TD>\n");

		printf("</TR>\n");
	        }

	printf("</TABLE>\n");
	printf("</DIV>\n");
	printf("</P>\n");

	return;
        }



void display_serviceescalations(void){
	serviceescalation *temp_se=NULL;
	contactsmember *temp_contactsmember=NULL;
	contactgroupsmember *temp_contactgroupsmember=NULL;
	char time_string[16]="";
	int options=FALSE;
	int odd=0;
	char *bg_class="";
	int contact=0;

	/* see if user is authorized to view hostgroup information... */
	if(is_authorized_for_configuration_information(&current_authdata)==FALSE){
		unauthorized_message();
		return;
	        }

	printf("<P><DIV ALIGN=CENTER CLASS='dataTitle'>サービスエスカレーション</DIV></P>\n");

	printf("<P>\n");
	printf("<DIV ALIGN=CENTER>\n");

	printf("<TABLE BORDER=0 CLASS='data'>\n");
	printf("<TR>\n");
	printf("<TH CLASS='data' COLSPAN=2>サービス</TH>");
	printf("</TR>\n");
	printf("<TR>\n");
	printf("<TH CLASS='data'>ホスト</TH>");
	printf("<TH CLASS='data'>説明</TH>");
	printf("<TH CLASS='data'>通知先グループ</TH>");
	printf("<TH CLASS='data'>初回通知</TH>");
	printf("<TH CLASS='data'>最終通知</TH>");
	printf("<TH CLASS='data'>通知間隔</TH>");
	printf("<TH CLASS='data'>エスカレーション期間</TH>");
	printf("<TH CLASS='data'>エスカレーションオプション</TH>");
	printf("</TR>\n");

	/* check all the service escalations... */
	for(temp_se=serviceescalation_list;temp_se!=NULL;temp_se=temp_se->next){

		if(odd){
			odd=0;
			bg_class="dataOdd";
		        }
		else{
			odd=1;
			bg_class="dataEven";
		        }

		printf("<TR CLASS='%s'>\n",bg_class);

		printf("<TD CLASS='%s'><A HREF='%s?type=hosts#%s'>%s</A></TD>",bg_class,CONFIG_CGI,url_encode(temp_se->host_name),temp_se->host_name);

		printf("<TD CLASS='%s'><A HREF='%s?type=services#%s;",bg_class,CONFIG_CGI,url_encode(temp_se->host_name));
		printf("%s'>%s</A></TD>\n",url_encode(temp_se->description),temp_se->description);

		printf("<TD CLASS='%s'>",bg_class);
		contact=0;
		for(temp_contactsmember=temp_se->contacts;temp_contactsmember!=NULL;temp_contactsmember=temp_contactsmember->next){
			contact++;
			if(contact>1)
				printf(", ");
			printf("<A HREF='%s?type=contacts#%s'>%s</A>\n",CONFIG_CGI,url_encode(temp_contactsmember->contact_name),temp_contactsmember->contact_name);
		        }
		for(temp_contactgroupsmember=temp_se->contact_groups;temp_contactgroupsmember!=NULL;temp_contactgroupsmember=temp_contactgroupsmember->next){
			contact++;
			if(contact>1)
				printf(", ");
			printf("<A HREF='%s?type=contactgroups#%s'>%s</A>\n",CONFIG_CGI,url_encode(temp_contactgroupsmember->group_name),temp_contactgroupsmember->group_name);
			}
		if(contact==0)
			printf("&nbsp;");
		printf("</TD>\n");

		printf("<TD CLASS='%s'>%d</TD>",bg_class,temp_se->first_notification);

		printf("<TD CLASS='%s'>",bg_class);
		if(temp_se->last_notification==0)
			printf("無限");
		else
			printf("%d",temp_se->last_notification);
		printf("</TD>\n");

		get_interval_time_string(temp_se->notification_interval,time_string,sizeof(time_string));
		printf("<TD CLASS='%s'>",bg_class);
		if(temp_se->notification_interval==0.0)
			printf("1回のみ通知(再通知無し)");
		else
			printf("%s",time_string);
		printf("</TD>\n");

		printf("<TD CLASS='%s'>",bg_class);
		if(temp_se->escalation_period==NULL)
			printf("&nbsp;");
		else
			printf("<A HREF='%s?type=timeperiods#%s'>%s</A>",CONFIG_CGI,url_encode(temp_se->escalation_period),temp_se->escalation_period);
		printf("</TD>\n");

		printf("<TD CLASS='%s'>",bg_class);
		options=FALSE;
		if(temp_se->escalate_on_warning==TRUE){
			printf("%s警告",(options==TRUE)?", ":"");
			options=TRUE;
		        }
		if(temp_se->escalate_on_unknown==TRUE){
			printf("%s不明",(options==TRUE)?", ":"");
			options=TRUE;
		        }
		if(temp_se->escalate_on_critical==TRUE){
			printf("%s障害",(options==TRUE)?", ":"");
			options=TRUE;
		        }
		if(temp_se->escalate_on_recovery==TRUE){
			printf("%s復旧",(options==TRUE)?", ":"");
			options=TRUE;
		        }
		if(options==FALSE)
			printf("無し");
		printf("</TD>\n");

		printf("</TR>\n");
	        }

	printf("</TABLE>\n");
	printf("</DIV>\n");
	printf("</P>\n");

	return;
        }



void display_hostdependencies(void){
	hostdependency *temp_hd;
	int odd=0;
	int options;
	char *bg_class="";

	/* see if user is authorized to view hostdependency information... */
	if(is_authorized_for_configuration_information(&current_authdata)==FALSE){
		unauthorized_message();
		return;
	        }

	printf("<P><DIV ALIGN=CENTER CLASS='dataTitle'>ホスト依存関係</DIV></P>\n");

	printf("<P>\n");
	printf("<DIV ALIGN=CENTER>\n");

	printf("<TABLE BORDER=0 CLASS='data'>\n");
	printf("<TR>\n");
	printf("<TH CLASS='data'>依存ホスト</TH>");
	printf("<TH CLASS='data'>親ホスト</TH>");
	printf("<TH CLASS='data'>依存タイプ</TH>");
	printf("<TH CLASS='data'>依存期間</TH>");
	printf("<TH CLASS='data'>依存失敗オプション</TH>");
	printf("</TR>\n");

	/* check all the host dependencies... */
	for(temp_hd=hostdependency_list;temp_hd!=NULL;temp_hd=temp_hd->next){

		if(odd){
			odd=0;
			bg_class="dataOdd";
		        }
		else{
			odd=1;
			bg_class="dataEven";
		        }

		printf("<TR CLASS='%s'>\n",bg_class);

		printf("<TD CLASS='%s'><A HREF='%s?type=hosts#%s'>%s</A></TD>",bg_class,CONFIG_CGI,url_encode(temp_hd->dependent_host_name),temp_hd->dependent_host_name);

		printf("<TD CLASS='%s'><A HREF='%s?type=hosts#%s'>%s</A></TD>",bg_class,CONFIG_CGI,url_encode(temp_hd->host_name),temp_hd->host_name);

		printf("<TD CLASS='%s'>%s</TD>",bg_class,(temp_hd->dependency_type==NOTIFICATION_DEPENDENCY)?"通知":"チェック実行");

		printf("<TD CLASS='%s'>",bg_class);
		if(temp_hd->dependency_period==NULL)
			printf("&nbsp;");
		else
			printf("<A HREF='%s?type=timeperiods#%s'>%s</A>",CONFIG_CGI,url_encode(temp_hd->dependency_period),temp_hd->dependency_period);
		printf("</TD>\n");

		printf("<TD CLASS='%s'>",bg_class);
		options=FALSE;
		if(temp_hd->fail_on_up==TRUE){
			printf("稼動");
			options=TRUE;
		        }
		if(temp_hd->fail_on_down==TRUE){
			printf("%s停止",(options==TRUE)?", ":"");
			options=TRUE;
		        }
		if(temp_hd->fail_on_unreachable==TRUE){
			printf("%s未到達",(options==TRUE)?", ":"");
			options=TRUE;
		        }
		if(temp_hd->fail_on_pending==TRUE){
			printf("%s未解決",(options==TRUE)?", ":"");
			options=TRUE;
		        }
		printf("</TD>\n");

		printf("</TR>\n");
	        }

	printf("</TABLE>\n");
	printf("</DIV>\n");
	printf("</P>\n");

	return;
        }



void display_hostescalations(void){
	hostescalation *temp_he=NULL;
	contactsmember *temp_contactsmember=NULL;
	contactgroupsmember *temp_contactgroupsmember=NULL;
	char time_string[16]="";
	int options=FALSE;
	int odd=0;
	char *bg_class="";
	int contact=0;

	/* see if user is authorized to view hostgroup information... */
	if(is_authorized_for_configuration_information(&current_authdata)==FALSE){
		unauthorized_message();
		return;
	        }

	printf("<P><DIV ALIGN=CENTER CLASS='dataTitle'>ホストエスカレーション</DIV></P>\n");

	printf("<P>\n");
	printf("<DIV ALIGN=CENTER>\n");

	printf("<TABLE BORDER=0 CLASS='data'>\n");
	printf("<TR>\n");
	printf("<TH CLASS='data'>ホスト</TH>");
	printf("<TH CLASS='data'>通知先グループ</TH>");
	printf("<TH CLASS='data'>初回通知</TH>");
	printf("<TH CLASS='data'>最終通知</TH>");
	printf("<TH CLASS='data'>通知間隔</TH>");
	printf("<TH CLASS='data'>エスカレーション期間</TH>");
	printf("<TH CLASS='data'>エスカレーションオプション</TH>");
	printf("</TR>\n");

	/* check all the host escalations... */
	for(temp_he=hostescalation_list;temp_he!=NULL;temp_he=temp_he->next){

		if(odd){
			odd=0;
			bg_class="dataOdd";
		        }
		else{
			odd=1;
			bg_class="dataEven";
		        }

		printf("<TR CLASS='%s'>\n",bg_class);

		printf("<TD CLASS='%s'><A HREF='%s?type=hosts#%s'>%s</A></TD>",bg_class,CONFIG_CGI,url_encode(temp_he->host_name),temp_he->host_name);

		printf("<TD CLASS='%s'>",bg_class);
		contact=0;
		for(temp_contactsmember=temp_he->contacts;temp_contactsmember!=NULL;temp_contactsmember=temp_contactsmember->next){
			contact++;
			if(contact>1)
				printf(", ");
			printf("<A HREF='%s?type=contacts#%s'>%s</A>\n",CONFIG_CGI,url_encode(temp_contactsmember->contact_name),temp_contactsmember->contact_name);
		        }
		for(temp_contactgroupsmember=temp_he->contact_groups;temp_contactgroupsmember!=NULL;temp_contactgroupsmember=temp_contactgroupsmember->next){
			contact++;
			if(contact>1)
				printf(", ");
			printf("<A HREF='%s?type=contactgroups#%s'>%s</A>\n",CONFIG_CGI,url_encode(temp_contactgroupsmember->group_name),temp_contactgroupsmember->group_name);
			}
		if(contact==0)
			printf("&nbsp;");
		printf("</TD>\n");

		printf("<TD CLASS='%s'>%d</TD>",bg_class,temp_he->first_notification);

		printf("<TD CLASS='%s'>",bg_class);
		if(temp_he->last_notification==0)
			printf("無限");
		else
			printf("%d",temp_he->last_notification);
		printf("</TD>\n");

		get_interval_time_string(temp_he->notification_interval,time_string,sizeof(time_string));
		printf("<TD CLASS='%s'>",bg_class);
		if(temp_he->notification_interval==0.0)
			printf("1回のみ(再通知無し)");
		else
			printf("%s",time_string);
		printf("</TD>\n");

		printf("<TD CLASS='%s'>",bg_class);
		if(temp_he->escalation_period==NULL)
			printf("&nbsp;");
		else
			printf("<A HREF='%s?type=timeperiods#%s'>%s</A>",CONFIG_CGI,url_encode(temp_he->escalation_period),temp_he->escalation_period);
		printf("</TD>\n");

		printf("<TD CLASS='%s'>",bg_class);
		options=FALSE;
		if(temp_he->escalate_on_down==TRUE){
			printf("%s停止",(options==TRUE)?", ":"");
			options=TRUE;
		        }
		if(temp_he->escalate_on_unreachable==TRUE){
			printf("%s未到達",(options==TRUE)?", ":"");
			options=TRUE;
		        }
		if(temp_he->escalate_on_recovery==TRUE){
			printf("%s復旧",(options==TRUE)?", ":"");
			options=TRUE;
		        }
		if(options==FALSE)
			printf("無し");
		printf("</TD>\n");

		printf("</TR>\n");
	        }

	printf("</TABLE>\n");
	printf("</DIV>\n");
	printf("</P>\n");

	return;
        }



void unauthorized_message(void){

	printf("<P><DIV CLASS='errorMessage'>設定情報を見る権限が無いようです。</DIV></P>\n");
	printf("<P><DIV CLASS='errorDescription'>このメッセージが何らかのエラーであると思う場合はHTTPサーバのこのCGIに対するアクセス権限の設定かNagiosのCGI用設定ファイルの認証に関するオプションを調べてみてください。</DIV></P>\n");

	return;
	}




void display_options(void){

	printf("<br><br>\n");

	printf("<div align=center class='reportSelectTitle'>閲覧したい設定項目を選んでください。</div>\n");

	printf("<br><br>\n");

        printf("<form method=\"get\" action=\"%s\">\n",CONFIG_CGI);

	printf("<div align=center>\n");
	printf("<table border=0>\n");

	printf("<tr><td align=left class='reportSelectSubTitle'>オブジェクトタイプ:</td></tr>\n");
	printf("<tr><td align=left class='reportSelectItem'>");
	printf("<select name='type'>\n");
	printf("<option value='hosts' %s>ホスト\n",(display_type==DISPLAY_HOSTS)?"SELECTED":"");
	printf("<option value='hostdependencies' %s>ホスト依存関係\n",(display_type==DISPLAY_HOSTDEPENDENCIES)?"SELECTED":"");
	printf("<option value='hostescalations' %s>ホストエスカレーション\n",(display_type==DISPLAY_HOSTESCALATIONS)?"SELECTED":"");
	printf("<option value='hostgroups' %s>ホストグループ\n",(display_type==DISPLAY_HOSTGROUPS)?"SELECTED":"");
	printf("<option value='services' %s>サービス\n",(display_type==DISPLAY_SERVICES)?"SELECTED":"");
	printf("<option value='servicegroups' %s>サービスグループ\n",(display_type==DISPLAY_SERVICEGROUPS)?"SELECTED":"");
	printf("<option value='servicedependencies' %s>サービス依存関係\n",(display_type==DISPLAY_SERVICEDEPENDENCIES)?"SELECTED":"");
	printf("<option value='serviceescalations' %s>サービスエスカレーション\n",(display_type==DISPLAY_SERVICEESCALATIONS)?"SELECTED":"");
	printf("<option value='contacts' %s>通知先\n",(display_type==DISPLAY_CONTACTS)?"SELECTED":"");
	printf("<option value='contactgroups' %s>通知先グループ\n",(display_type==DISPLAY_CONTACTGROUPS)?"SELECTED":"");
	printf("<option value='timeperiods' %s>時間帯\n",(display_type==DISPLAY_TIMEPERIODS)?"SELECTED":"");
	printf("<option value='commands' %s>コマンド\n",(display_type==DISPLAY_COMMANDS)?"SELECTED":"");
	printf("</select>\n");
	printf("</td></tr>\n");

	printf("<tr><td class='reportSelectItem'><input type='submit' value='続行'></td></tr>\n");
	printf("</table>\n");
	printf("</div>\n");

	printf("</form>\n");

	return;
        }
